@@ -17,6 +17,7 @@ import collections
 import datetime
 import logging
 import os
+import subprocess
 import tempfile
 
 from google.cloud import ndb
@@ -245,63 +246,36 @@ class RepoAnalyzer:
     # TODO(ochang): Possibly look at commit message, author etc.
     return None
 
-  def contains_commit(self, repo, to_search, target_commit, _search_cache=None):  # pylint: disable=invalid-name
-    """Check if a reference contains a commit."""
-    target = repo.revparse_single(target_commit)
-    search = repo.revparse_single(to_search)
-
-    try:
-      commits = repo.walk(search.id)
-    except ValueError:
-      # Invalid commit
-      return False
-
-    result = False
-    walked = []
-    for commit in commits:
-      if _search_cache:
-        cached_result = _search_cache.get(str(commit.id))
-        if cached_result is not None:
-          result = cached_result
-          break
-
-      walked.append(str(commit.id))
-
-      if commit.id == target.id:
-        result = True
-        break
-
-    if _search_cache is not None:
-      for entry in walked:
-        _search_cache[entry] = result
-
-    return result
-
   def get_tags_with_commits(self, repo, commits):
     """Get tags with a given commit."""
     if not commits:
       return set()
 
-    affected = set()
     logging.info('Getting tags which contain %s', ','.join(commits))
 
     tags = [
         ref for ref in repo.listall_references() if ref.startswith(TAG_PREFIX)
     ]
 
-    search_caches = {commit: {} for commit in commits}
-    for tag in tags:
-      if self.detect_cherrypicks:
+    if self.detect_cherrypicks:
+      affected = set()
+      for tag in tags:
         if all(
             self.get_equivalent_commit(repo, tag, commit)
             for commit in commits):
           affected.add(tag[len(TAG_PREFIX):])
-      else:
-        if all(
-            self.contains_commit(
-                repo, tag, commit, _search_cache=search_caches[commit])
-            for commit in commits):
-          affected.add(tag[len(TAG_PREFIX):])
+    else:
+      affected = None
+      # pygit2 does not provide any easy to way to compute this efficiently, so
+      # we use git here.
+      for commit in commits:
+        current = subprocess.check_output(
+            ['git', '-C', repo.path, 'tag', '--contains',
+             commit]).decode().splitlines()
+        if affected is None:
+          affected = set(current)
+        else:
+          affected &= set(current)
 
     return affected
 
